@echo off

if exist C:\cygwin\bin\*.* set CYGWIN_PATH=C:\cygwin\bin
if exist D:\cygwin\bin\*.* set CYGWIN_PATH=D:\cygwin\bin

if exist C:\MinGW\bin\*.* set MINGW_PATH=C:\MinGW\bin
if exist D:\MinGW\bin\*.* set MINGW_PATH=D:\MinGW\bin

rem Test if Visual Studio 2005 is installed
if exist "%ProgramFiles%\Microsoft Visual Studio 8\VC\*.*" set VC_PATH=%ProgramFiles%\Microsoft Visual Studio 8\VC\
if exist "%ProgramFiles(x86)%\Microsoft Visual Studio 8\VC\*.*" set VC_PATH=%ProgramFiles(x86)%\Microsoft Visual Studio 8\VC\

rem Test if Visual Studio 2008 is installed
if exist "%ProgramFiles%\Microsoft Visual Studio 9.0\VC\*.*" set VC_PATH=%ProgramFiles%\Microsoft Visual Studio 9.0\VC\
if exist "%ProgramFiles(x86)%\Microsoft Visual Studio 9.0\VC\*.*" set VC_PATH=%ProgramFiles(x86)%\Microsoft Visual Studio 9.0\VC\

rem Test if Visual Studio 2010 is installed
if exist "%ProgramFiles%\Microsoft Visual Studio 10.0\VC\*.*" set VC_PATH=%ProgramFiles%\Microsoft Visual Studio 10.0\VC\
if exist "%ProgramFiles(x86)%\Microsoft Visual Studio 10.0\VC\*.*" set VC_PATH=%ProgramFiles(x86)%\Microsoft Visual Studio 10.0\VC\

rem Test if Visual Studio 2012 is installed
if exist "%ProgramFiles%\Microsoft Visual Studio 11.0\VC\*.*" set VC_PATH=%ProgramFiles%\Microsoft Visual Studio 11.0\VC\
if exist "%ProgramFiles(x86)%\Microsoft Visual Studio 11.0\VC\*.*" set VC_PATH=%ProgramFiles(x86)%\Microsoft Visual Studio 11.0\VC\

rem Test if Visual Studio 2013 is installed
if exist "%ProgramFiles%\Microsoft Visual Studio 12.0\VC\*.*" set VC_PATH=%ProgramFiles%\Microsoft Visual Studio 12.0\VC\
if exist "%ProgramFiles(x86)%\Microsoft Visual Studio 12.0\VC\*.*" set VC_PATH=%ProgramFiles(x86)%\Microsoft Visual Studio 12.0\VC\

rem Test if Visual Studio 2015 is installed
if exist "%ProgramFiles%\Microsoft Visual Studio 14.0\VC\*.*" set VC_PATH=%ProgramFiles%\Microsoft Visual Studio 14.0\VC\
if exist "%ProgramFiles(x86)%\Microsoft Visual Studio 14.0\VC\*.*" set VC_PATH=%ProgramFiles(x86)%\Microsoft Visual Studio 14.0\VC\

@REM -----------------------------------------------------------------------

if "%1"=="g++-3" goto gpp3
if "%1"=="g3" goto gpp3
if "%1"=="g++-4" goto gpp4
if "%1"=="g4" goto gpp4
if "%1"=="msvc" goto msvc
if "%1"=="vc" goto msvc
if "%1"=="mgw" goto mgw

echo Usage: Makefile.cmd compiler [target] [static]
echo Compilers:
echo  g3, g++-3 - GNU 3 C++ compiler
echo  g4, g++-4 - GNU 4 C++ compiler (targets: x86, x64)
echo  mgw - MinGW C++ compiler (targets: x86)
echo  vc, msvc - Microsoft Visual C++ compiler (targets: x86, amd64, x86_amd64)
echo Options:
echo  static - use a precompiled static library

pause

exit /B

@REM -----------------------------------------------------------------------

:gpp3

echo Compile sample using g++-3 compiler

set PATH=%CYGWIN_PATH%
set CYGWIN=nodosfilewarning 

set BASE_DIR=../../../../
set LIB_DIR=%BASE_DIR%Libraries/All/Win32/
set SRC_DIR=%BASE_DIR%WrapperSources/

set CPP=g++-3
set CUSTOM_FLAGS=
set CUSTOM_LDFLAGS=-static-libgcc -Wl,--enable-stdcall-fixup -Wl,--gc-sections -Wl,--print-gc-sections -Wl,--strip-all
set CUSTOM_DEFINES=-DSB_DEFINE_WSTRING 

if "%3"=="static" (
  rem Using a precompiled static library
  set STATIC_LIB=sbbex3
  make -f Makefile_static.gpp
) else (
  make -f Makefile.gpp
)

exit /B

:gpp4

echo Compile sample using g++-4 compiler

set PATH=%CYGWIN_PATH%
set CYGWIN=nodosfilewarning 

set BASE_DIR=../../../../
set LIB_DIR=%BASE_DIR%Libraries/All/Win32/
set SRC_DIR=%BASE_DIR%WrapperSources/

set CPP=g++-4
set CUSTOM_FLAGS=-m32
set CUSTOM_LDFLAGS=-static-libgcc -Wl,--enable-stdcall-fixup -Wl,--gc-sections  -Wl,--print-gc-sections -Wl,--strip-all
set CUSTOM_DEFINES=
if "%2"=="x64" (
  set CUSTOM_FLAGS=-m64
  set CUSTOM_DEFINES=-DCPU64
  set LIB_DIR=%BASE_DIR%Libraries/All/Win64/
)

if "%3"=="static" (
  rem Using a precompiled static library
  set STATIC_LIB=sbbex4
  make -f Makefile_static.gpp
) else (
  make -f Makefile.gpp
)

exit /B

@REM -----------------------------------------------------------------------

:mgw

echo Compile static library using MinGW g++ compiler

set PATH=%MINGW_PATH%

set BASE_DIR=../../../../
set LIB_DIR=%BASE_DIR%Libraries/All/Win32/
set SRC_DIR=%BASE_DIR%WrapperSources/

set CPP=g++
set CUSTOM_FLAGS=-m32
set CUSTOM_LDFLAGS=-static-libgcc -Wl,--enable-stdcall-fixup -Wl,--gc-sections  -Wl,--print-gc-sections -Wl,--strip-all
set CUSTOM_DEFINES=

if "%3"=="static" (
  rem Using a precompiled static library
  set STATIC_LIB=sbbexmgw
  mingw32-make -f Makefile_static.mgw
) else (
  mingw32-make -f Makefile.mgw
)

exit /B

@REM -----------------------------------------------------------------------

:msvc

echo Compile sample using VC compiler

call "%VC_PATH%\vcvarsall.bat" %2

rem split LIB environment variable
setlocal ENABLEDELAYEDEXPANSION
set VCLIBPATH=
set list=%LIB%

:splitList
for /f "tokens=1* delims=;" %%a in ("!list!") do (
  if "%%a" NEQ "" set VCLIBPATH=!VCLIBPATH! "/LIBPATH:%%a"
  if "%%b" NEQ "" (
    set list=%%b
    goto splitList
  )
)

set BASE_DIR=..\..\..\..\
set LIB_DIR=%BASE_DIR%\Libraries\All\Win32\
set SRC_DIR=%BASE_DIR%WrapperSources\

set CUSTOM_DEFINES=
if "%2"=="amd64" (
  set CUSTOM_DEFINES=/D CPU64
  set LIB_DIR=%BASE_DIR%\Libraries\All\Win64\
)

if "%2"=="x86_amd64" (
  set CUSTOM_DEFINES=/D CPU64
  set LIB_DIR=%BASE_DIR%\Libraries\All\Win64\
  set CUSTOM_LDFLAGS=/machine:x64
)

if "%3"=="static" (
  rem Using a precompiled static library
  nmake -f Makefile_static.vc
) else (
  nmake -f Makefile.vc
)

exit /B
