#include <cstring>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <algorithm>
#include <ctime>

#include "sbutils.h"
#include "sbmisc.h"
#include "sbhttpcrl.h"
#include "sbhttpsclient.h"
#include "sbmisc.h"

using namespace SecureBlackbox;

std::string UnQuote(std::string s)
{
    if (s.length() < 1)
        return s;

	s.erase(remove( s.begin(), s.end(), '\"' ), s.end());

    return s;
}

void SB_CALLBACK OnProgress(void * /*_ObjectData*/, TObjectHandle /*Sender*/, int64_t Total, int64_t Current, int8_t * Cancel)
{
	if (Total > 0)
	{
		int percent = (Current * 100) / Total;
		std::cout << percent << "%" << std::endl;
	}
}

void SB_CALLBACK OnCertificateValidate(void * /*_ObjectData*/, TObjectHandle /*Sender*/, TElX509CertificateHandle X509Certificate, int8_t * Validate)
{
	*Validate = true; 
	// NEVER do this in production code. Instead perform full certificate validation.
	// You can use TElX509CertificateValidator class for complete certificiate chain validation. 
	// See other samples for example of how TElX509CertificateValidator class should be used.
}

void SB_CALLBACK OnDocumentBegin(void * /*_ObjectData*/, TObjectHandle /*Sender*/)
{
	std::cout << "-- Document started --" << std::endl;
}

void SB_CALLBACK OnDocumentEnd(void * /*_ObjectData*/, TObjectHandle /*Sender*/)
{
	std::cout << "-- Document finished --" << std::endl;
}

void SB_CALLBACK OnCloseConnection(void * /*_ObjectData*/, TObjectHandle /*Sender*/, TSBCloseReasonRaw CloseReason)
{
	std::cout << "-- Connection closed --" << std::endl;
}

void SB_CALLBACK OnData(void * /*_ObjectData*/, TObjectHandle /*Sender*/, void * Buffer, int32_t Size)
{
	std::string data((char*)Buffer, Size);
	std::cout << data << std::endl;
}

int main(int argc, char **argv)
{
	int i;
	std::string helpMessage = "consoleHTTPSPost.exe [params]\n" \
		"[params]\n" \
		"-url : location of the script to upload (default : http://localhost/upload/simple_upload.php )\n" \
		"-in : file to upload\n";
	std::string url = "http://localhost/upload/simple_upload.php";
	std::string inputFilename = "";

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << std::endl;
		std::cout << E.what() << std::endl;
		return 0;
	}

	if (argc < 2)
	{
		std::cout << helpMessage << std::endl;
		return 0;
	}

	std::cout << "EldoS HTTPS post client console demo." << std::endl;

	try
	{
		i = 1;
		while (i < argc)
		{
			std::string param;
			param.assign(argv[i]);

			if (param == "-url")
			{
				if (++i <= argc)
					url = UnQuote(argv[i]);
			}
			else if (param == "-in")
			{
				if (++i <= argc)
					inputFilename = UnQuote(argv[i]);
			}

			i++;
		}
	}
	catch (...)
	{
		std::cout << "Failed to parse parameters" << std::endl;
		std::cout << helpMessage << std::endl;
		return 0;
	}

	try
	{
		TElHTTPSClient HTTPSClient(NULL);
		if ((inputFilename != "") && (url != ""))
		{
			HTTPSClient.set_OnProgress(&OnProgress, NULL);
			HTTPSClient.set_OnCertificateValidate(&OnCertificateValidate, NULL);

			HTTPSClient.set_OnDocumentBegin(&OnDocumentBegin, NULL);
			HTTPSClient.set_OnDocumentEnd(&OnDocumentEnd, NULL);
			HTTPSClient.set_OnCloseConnection(&OnCloseConnection, NULL);
			HTTPSClient.set_OnData(&OnData, NULL);

			TFileStream stream(inputFilename, filemodeOpenRead);

			TElStringList SL;
			SL.Add("Upload");

			try
			{
				std::cout << "Uploading: " << inputFilename << std::endl;
				HTTPSClient.PostWebForm(url, SL, "userfile", inputFilename, stream, "", false);
			}
			catch (SBException E)
			{
				std::cout << "Exception happened during HTTP post: " << E.what() << std::endl;
			}
		}
	}
	catch (SBException ex)
	{
		std::cout << "Error" << ex.what() << std::endl;
		return 0;
	}
}