#include <cstring>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <algorithm>
#include <ctime>

#include "ServerThread.h"
#include "sbutils.h"
#include "sbstrutils.h"
#include "sbsocket.h"
#include "sbhttpsserver.h"
#include "sbhttpscommon.h"
#include "sbtypes.h"
#include "sbx509.h"
#include "sbcustomcertstorage.h"
#include "sbsharedresource.h"
#include "sbmisc.h"

using namespace SecureBlackbox;

TFileStream *FLog = NULL;
TElSharedResource *FSharedResource = NULL;

void OnLog(std::string Line)
{
    time_t timer = (time_t)DateTimeNow();
    char *dt = ctime(&timer);
    std::string str = "Time: ";
    str += dt;
    StringSubstring(str, 0, (int)str.length() - 1, str);
    str += " | Event: " + Line + "\n\n";
    FSharedResource->WaitToWrite();
    try
    {
        FLog->set_Position(FLog->get_Size());
        FLog->Write(static_cast<void*>(&str[0]), str.length());
    }
    catch (...) {}
    FSharedResource->Done();
}

void OnRequest(TElHTTPServerRequestParams *Request)
{
    std::string urlstr, mstr;
    switch (Request->get_Method()) {
        case hmGet: mstr = "GET"; break;
        case hmPost: mstr = "POST"; break;
        case hmHead: mstr = "HEAD"; break;
        case hmOptions: mstr = "OPTIONS"; break;
        case hmDelete: mstr = "DELETE"; break;
        case hmTrace: mstr = "TRACE"; break;
        case hmPut: mstr = "PUT"; break;
        case hmConnect: mstr = "CONNECT"; break;
        case hmCustom: Request->get_CustomMethod(mstr); break;
    }
    Request->get_URL(urlstr);
    std::string str = "Method: " + mstr + " | URL: " + urlstr + "\n\n";
    FSharedResource->WaitToWrite();
    try
    {
        FLog->set_Position(FLog->get_Size());
        FLog->Write(static_cast<void*>(&str[0]), str.length());
    }
    catch (...) {}
    FSharedResource->Done();
}

int main(int argc, char **argv)
{
	int i;
	std::string helpMessage = "ConsoleHTTPServer.exe [params]\n" \
		"[params]\n" \
		"-port : server port (default : 8080);\n" \
		"-SSL : using SSL (default : N);\n" \
		"-compression : using compression (default : Y);\n" \
		"-chunking : using chunking (default : Y);\n";
	int port = 8080;
	bool ssl = false;
	bool compr = true;
	bool chunk = true;

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
        SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << std::endl;
		std::cout << E.what() << std::endl;
		return 0;
	}

	try
	{
		i = 1;
		while (i < argc)
		{
			std::string param;
			param.assign(argv[i]);

			if (param == "-port")
			{
				if (++i <= argc)
                {
                    port = std::atoi(argv[i]);
                }
			}
			else if (param == "-SSL")
			{
				if (++i <= argc)
                {
                    std::string value = argv[i];
                    std::transform(value.begin(), value.end(), value.begin(), ::toupper);
                    
                    if ((value == "Y") || (value == "YES"))
                    {
                        ssl = true;
                    }
                    else if ((value == "N") || (value == "NO"))
                    {
                        ssl = false;
                    }
                    else
                    {
                        std::cout << "Wrong SSL parameter" << std::endl;
                        return 0;
                    }
                }
			}
			else if (param == "-compression")
			{
				if (++i <= argc)
                {
                    std::string value = argv[i];
                    std::transform(value.begin(), value.end(), value.begin(), ::toupper);
                    
                    if ((value == "Y") || (value == "YES"))
                    {
                        compr = true;
                    }
                    else if ((value == "N") || (value == "NO"))
                    {
                        compr = false;
                    }
                    else
                    {
                        std::cout << "Wrong Compression parameter" << std::endl;
                        return 0;
                    }
                }
			}
			else if (param == "-chunking")
			{
				if (++i <= argc)
                {
                    std::string value = argv[i];
                    std::transform(value.begin(), value.end(), value.begin(), ::toupper);
                    
                    if ((value == "Y") || (value == "YES"))
                    {
                        chunk = true;
                    }
                    else if ((value == "N") || (value == "NO"))
                    {
                        chunk = false;
                    }
                    else
                    {
                        std::cout << "Wrong Chunking parameter" << std::endl;
                        return 0;
                    }
                }
			}

			i++;
		}
	}
	catch (...)
	{
		std::cout << "Failed to parse parameters" << std::endl;
		std::cout << helpMessage << std::endl;
		return 0;
	}

	try
	{
        std::string basePath;
        SBExtractFilePath(argv[0], basePath);
        std::string certPath = basePath + "/cert.pem";
        
        TElMemoryCertStorage *storage = NULL;
        if (ssl)
        {
            storage = new TElMemoryCertStorage(NULL);
            TElX509Certificate *cert = new TElX509Certificate(NULL);
            try
            {
                int res = cert->LoadFromFileAuto(certPath, "password");
                if (res == 0)
                {
                    storage->Add(cert, true);
                }
                else
                {
                    std::cout << "Can't load server certificate, error " << res << std::endl;
                    return 0;
                }
            }
            catch (SBException E)
            {
                std::cout << "Error " << E.what() << std::endl;
                return 0;
            }
        }
        
        THTTPSServerListeningThread *FServerThread = NULL;
        try
        {
            FSharedResource = new TElSharedResource();
            FLog = new TFileStream(basePath + "/serverLog.txt", fmOpenReadWrite);
            
            FServerThread = new THTTPSServerListeningThread(port);
            FServerThread->FOnLog = OnLog;
            FServerThread->FOnRequest = OnRequest;
            FServerThread->FBasePath = basePath;
            FServerThread->FSSLMode = ssl;
            FServerThread->FUseCompression = compr;
            FServerThread->FUseChunkedTransfer = chunk;
            if (ssl && storage)
                FServerThread->FCertStorage = storage;
            FServerThread->Start();
            OnLog("Server started");
        }
        catch (SBException E)
        {
            FLog->Free();
            std::cout << "Error " << E.what() << std::endl;
            return 0;
        }
        
        std::cout << "HTTPServer is working now, log - " << basePath << "/serverLog.txt" << std::endl;
        std::cout << "...Just press key 'S' to stop HTTPServer..." << std::endl;
        
        bool shouldContinue = true;
        do {
            char input;
            std::cin >> input;
            if ((input == 's') || (input == 'S')) shouldContinue = false;
        } while (shouldContinue);
        
        FServerThread->Terminate();
        while (true) {
            if (FServerThread->slFinished) {
                delete FServerThread;
                break;
            }
        }
        OnLog("Server stopped");
        
        if (FLog) delete FLog;
        if (FSharedResource) delete FSharedResource;
        
        std::cout << "Server stopped. BYE" << std::endl;
    }
	catch (SBException ex)
	{
		std::cout << "Error " << ex.what() << std::endl;
		return 0;
	}
}