//
//  ServerThread.cpp
//  OSXHTTPServer
//
//  Created by EldoS on 13.03.15.
//  Copyright (c) 2015 EldoS. All rights reserved.
//

#include "ServerThread.h"
#include "sbmisc.h"

// This procedure can be used to compress server response in order to calculate and set its CONTENT-LENGTH header
void CompressStream(TStream *Input, TStream *Output, TSBHTTPCompressionAlg CompAlg, int CompLevel)
{
    TElHTTPCompressedProcessor *Comp = new TElHTTPCompressedProcessor();
    Comp->set_CompressionLevel(CompLevel);
    
    if (CompAlg == hcaGZip) Comp->set_Gzipped(true);
    else Comp->set_Deflated(true);
    
    Comp->Initialize(true);
    try {
        int size = 102400;
        std::vector<uint8_t> Buf(size);
        int Count;
        do {
            Count = Input->Read(static_cast<void*>(&Buf[0]), size);
            if (Count > 0) {
                std::vector<uint8_t> Tmp;
                Comp->EncodeData(Buf, 0, Count, Tmp);
                Output->Write(static_cast<void*>(&Tmp[0]), (int)Tmp.size());
            }
        } while (Count > 0);
        
        Comp->Reset();
    } catch (SBException E) {
        Comp->Reset();
    }
}

std::string URLExtractPath(std::string URL)
{
    std::string Protocol, Username, Password, Host, Path, Anch, Params;
    uint16_t Port;
    if ((URL.length() > 0) && (URL[0] == '/')) return URL;
    else {
        ParseURL(URL, true, Protocol, Username, Password, Host, Port, Path, Anch, Params);
        return Path;
    }
}

std::string URLRemoveSlash(std::string URL)
{
    if ((URL.length() > 0) && (URL != "/") && ((URL[URL.length() - 1] == '/') || (URL[URL.length() - 1] == '\\'))) {
        std::string res;
        StringSubstring(URL, 0, (int)URL.length() - 1, res);
        return res;
    }
    else return URL;
}

std::string BoolToStr(bool Value)
{
    if (Value) return "true";
    else return "false";
}

THTTPSServerListeningThread::THTTPSServerListeningThread(int ListenPort)
{
    slTerminated = false;
    slFinished = false;

    FOnLog = NULL;
    FOnRequest = NULL;
    
    FSharedResource = new TElSharedResource();
    FSessions = new TList();
    FServerSocket = new TElSocket(NULL);
    FServerSocket->set_ListenPort(ListenPort);
    FCertStorage = NULL;
}

THTTPSServerListeningThread::~THTTPSServerListeningThread()
{
    FSharedResource->WaitToWrite();
    try
    {
        for (int i = 0; i < FSessions->get_Count(); i++) {
            ((THTTPSServerThread*)FSessions->get_Items(i))->Terminate();
        }
        FSharedResource->Done();
    }
    catch (...)
    {
        FSharedResource->Done();
    }

    // waiting session threads to terminate
    while (true)
    {
        FSharedResource->WaitToRead();
        try
        {
            if (FSessions->get_Count() == 0)
                break;
            else {
                for (int i = 0; i < FSessions->get_Count(); i++) {
                    if (((THTTPSServerThread*)FSessions->get_Items(i))->sFinished) {
                        delete ((THTTPSServerThread*)FSessions->get_Items(i));
                    }
                }
            }
            FSharedResource->Done();
        }
        catch (...)
        {
            FSharedResource->Done();
        }
        std::this_thread::sleep_for(std::chrono::milliseconds(10));
    }
    FSharedResource->Done();
    
    delete FServerSocket;
    delete FSharedResource;
    delete FSessions;
}

void THTTPSServerListeningThread::Terminate()
{
    slTerminated = true;
    if (slThread.joinable()) slThread.join();
}

void THTTPSServerListeningThread::Start()
{
    slThread = std::thread(&THTTPSServerListeningThread::Execute, this);
}

void THTTPSServerListeningThread::Log(std::string Msg)
{
    if ((FOnLog) && (Msg != "")) FOnLog(Msg);
}

void THTTPSServerListeningThread::Execute()
{
    int Res = FServerSocket->Bind();
    if (Res != 0)
    {
        Log("Failed to bind socket to the port");
        slFinished = true;
        return;
    }

    Res = FServerSocket->Listen();
    if (Res != 0)
    {
        Log("Failed to bind socket to the port");
        slFinished = true;
        return;
    }

    try
    {
        while (!slTerminated)
        {
            if (FSessions->get_Count() > 0)
            {
                for (int i = 0; i < FSessions->get_Count(); i++) {
                    if (((THTTPSServerThread*)FSessions->get_Items(i))->sFinished) {
                        if (((THTTPSServerThread*)(FSessions->get_Items(i)))->sThread.joinable())
                            ((THTTPSServerThread*)(FSessions->get_Items(i)))->sThread.join();

                        delete ((THTTPSServerThread*)(FSessions->get_Items(i)));
                    }
                }
            }
            
            TElSocket *Socket = new TElSocket(NULL);
            FServerSocket->Accept(100, *Socket);
            if (!Socket->getHandle())
            {
                std::this_thread::sleep_for(std::chrono::milliseconds(50));
                continue;
            }

            std::string addr;
            Socket->get_RemoteAddress(addr);
            Log("Client accepted: " + addr);

            THTTPSServerThread *ClientThread;
            FSharedResource->WaitToWrite();
            try
            {
                ClientThread = new THTTPSServerThread(this, Socket);
                ClientThread->set_BasePath(FBasePath);
                ClientThread->set_SSLMode(FSSLMode);
                if (FCertStorage) ClientThread->set_CertStorage(FCertStorage);
                ClientThread->FOnLog = FOnLog;
                ClientThread->FOnRequest = FOnRequest;
                ClientThread->FUseCompression = FUseCompression;
                ClientThread->FUseChunkedTransfer = FUseChunkedTransfer;
                
                FSessions->Add(ClientThread);

                FSharedResource->Done();
            }
            catch (...)
            {
                FSharedResource->Done();
            }

            ClientThread->Start();
        }
        FServerSocket->Close(true);
    }
    catch (...)
    {
        FServerSocket->Close(true);
    }
    slFinished = true;
}

THTTPSServerThread::THTTPSServerThread(THTTPSServerListeningThread *Parent, TElSocket *Socket)
{
    sTerminated = false;
    sFinished = false;
    
    FOnLog = NULL;
    FOnRequest = NULL;
    
    FParent = Parent;
    FSocket = Socket;
    FFile = NULL;
    FUploadStream = new TMemoryStream();
    FWSServer = NULL;
    FCurrentRequest = NULL;
    FCurrentResponse = NULL;
    
    FHTTPSServer = new TElHTTPSServer(NULL);
    // internal handlers
    FHTTPSServer->set_OnSend(DoSend, this);
    FHTTPSServer->set_OnReceive(DoReceive, this);
    FHTTPSServer->set_OnOpenConnection(DoOpenConnection, this);
    FHTTPSServer->set_OnCloseConnection(DoCloseConnection, this);
    FHTTPSServer->set_OnRequestHeadersReceived(DoRequestHeadersReceived, this);
    FHTTPSServer->set_OnData(DoData, this);
    FHTTPSServer->set_OnRequestBodyReceived(DoRequestBodyReceived, this);
    FHTTPSServer->set_OnSSLError(DoError, this);
    FHTTPSServer->set_OnBeforeWebSocketServerUsed(DoBeforeWebSocketServerUsed, this);
    FHTTPSServer->set_OnWebSocketConnectionEstablished(DoWebSocketConnectionEstablished, this);
}

THTTPSServerThread::~THTTPSServerThread()
{
    if (FFile) delete FFile;

    if (FHTTPSServer && (FHTTPSServer->get_Active())) FHTTPSServer->Close(false);

    delete FHTTPSServer;
    delete FSocket;
    delete FUploadStream;
    
    FParent->FSharedResource->WaitToWrite();
    try
    {
        FParent->FSessions->Remove(this);
        FParent->FSharedResource->Done();
    }
    catch (SBException E)
    {
        FParent->FSharedResource->Done();
    }
}

inline void THTTPSServerThread::set_BasePath(std::string value)
{
    FBasePath = URLRemoveSlash(value);
}

inline void THTTPSServerThread::set_CertStorage(TElCustomCertStorage *value)
{
    FHTTPSServer->set_CertStorage(value);
}

inline TElCustomCertStorage* THTTPSServerThread::get_CertStorage()
{
    return FHTTPSServer->get_CertStorage();
}

inline void THTTPSServerThread::set_SSLMode(bool value)
{
    FHTTPSServer->set_SSLMode(value);
}

inline bool THTTPSServerThread::get_SSLMode()
{
    return FHTTPSServer->get_SSLMode();
}

void THTTPSServerThread::Terminate()
{
    sTerminated = true;
    if (sThread.joinable()) sThread.join();
}

void THTTPSServerThread::Start()
{
    sThread = std::thread(&THTTPSServerThread::Execute, this);
}

void SaveFiles(TElMultipartFormList *List, std::string bPath, TMemoryStream *uStream)
{
    for (int i = 0; i < List->get_Count(); i++) {
        TElMultipartFormData *MFD = List->get_ValuesByIndex(i);
        try {
            TElMultipartFormList *MFL = new TElMultipartFormList(MFD->getHandle(), false);
            SaveFiles(MFL, bPath, uStream);
        } catch (SBException E) {
            std::string fname;
            MFD->get_FileName(fname);
            if (fname != "") {
                TFileStream *FS = new TFileStream(URLRemoveSlash(bPath) + "/" + fname, filemodeCreate);
                try {
                    uint8_t *Buffer = (uint8_t*)uStream->get_Memory();
                    int64_t Size = uStream->get_Size();
					Buffer += MFD->get_ContentStartPos();
					FS->Write(Buffer, MFD->get_ContentSize());
                    FS->Free();
                }
                catch (SBException E) {
                    FS->Free();
                }
            }
        }
    }
}

void THTTPSServerThread::ProcessRequest()
{
    try
    {
        TMemoryStream *MS = NULL;
        
        FCurrentResponse->set_UseChunkedTransfer(FUseChunkedTransfer);
        bool UseKeepAlive = FCurrentRequest->get_KeepAlive();
        FCurrentResponse->set_UseKeepAlive(UseKeepAlive);
        bool CompressionEnabled = FUseCompression;
        
        // process only GET requests
        if (FCurrentRequest->get_Method() == hmGet) {
            try {
                std::string reqURL;
                FCurrentRequest->get_URL(reqURL);
                FFile = new TFileStream(URLRemoveSlash(FBasePath) + reqURL, fmOpenRead | fmShareDenyNone);
                FCurrentResponse->set_StatusCode(200);
                
                // we should set content-length if chunked encoding is disabled
                if (!FCurrentResponse->get_UseChunkedTransfer()) {
                    if (!CompressionEnabled)
                        // we can calculate content-length because data is not compressed
                        FCurrentResponse->set_ContentLength(FFile->get_Size());
                    else {
                        MS = new TMemoryStream();
                        
                        if ((FCurrentRequest->get_PreferGzip()) && (FCurrentRequest->get_AcceptGZip())) {
                            CompressStream(FFile, MS, hcaGZip, 6);
                            FCurrentResponse->get_CustomHeaders()->Add("Content-Encoding: gzip");
                        }
                        else {
                            CompressStream(FFile, MS, hcaDeflate, 6);
                            FCurrentResponse->get_CustomHeaders()->Add("Content-Encoding: deflate");
                        }
                        
                        // Set content-length to compressed data size
                        FCurrentResponse->set_ContentLength(MS->get_Size());
                        MS->set_Position(0);
                    }
                }
            }
            catch (SBException E) {
                FCurrentResponse->set_StatusCode(404);
                FCurrentResponse->set_ContentLength(0);
            }
        }
        else if ((FCurrentRequest->get_Method() == hmPost) && FUpload) {
            try
            {
                std::string ContentType;
                FCurrentRequest->get_ContentType(ContentType);
                if (StringStartsWith(ContentType, "multipart/form-data")) {
                    FUploadStream->set_Position(0);
                    try {
                        TElMultipartFormList *MList = new TElMultipartFormList(FHTTPSServer->ParseMultipartFormData(FCurrentRequest, FUploadStream, false), false);
                        SaveFiles(MList, FBasePath, FUploadStream);
                    }
                    catch (SBException E) {
                        //for showing data that was send via TElHTTPSClient.Post methods
                        std::vector<uint8_t> Buf(FUploadStream->get_Size());
                        FUploadStream->set_Position(0);
                        FUploadStream->Read(static_cast<void*>(&Buf[0]), (int)Buf.size());
                        std::string str;
                        StringOfBytes(Buf, str);
                        Log("POST data received: " + str);
                    }
                }
                else {
                    std::vector<uint8_t> Buf(FUploadStream->get_Size());
                    FUploadStream->set_Position(0);
                    FUploadStream->Read(static_cast<void*>(&Buf[0]), (int)Buf.size());
                    
                    TElMultipartFormData *MFD = new TElMultipartFormData();
                    try {
                        MFD->Parse(Buf, 0, (int)Buf.size());
                        std::string fname;
                        MFD->get_FileName(fname);
                        if (fname != "") {
                            std::string fullname = URLRemoveSlash(FBasePath) + "/" + fname;
                            TFileStream *FS = new TFileStream(fullname, filemodeCreate);
                            try {
                                std::vector<uint8_t> content;
                                MFD->get_Content(content);
                                FS->Write(static_cast<void*>(&content[0]), (int)content.size());
                                FS->Free();
                            }
                            catch (SBException E) {
                                FS->Free();
                            }
                        }
                    }
                    catch (SBException E) {
                        //for showing data that was send via TElHTTPSClient.Post methods
                        std::string str;
                        StringOfBytes(Buf, str);
                        Log("POST data received: " + str);
                    }
                }
                
                // redirect
                FCurrentResponse->set_StatusCode(303);
                FCurrentResponse->set_Location("/upload.html");
                FCurrentResponse->set_ContentLength(0);
            }
            catch (SBException E) {
                FCurrentResponse->set_StatusCode(404);
                FCurrentResponse->set_ContentLength(0);
            }
        }
        else {
            FCurrentResponse->set_StatusCode(400);
            FCurrentResponse->set_ContentLength(0);
        }
        
        if (FFile)
            // turn off internal server compression if chunked encoding is disabled
            FCurrentResponse->set_UseCompression(FUseCompression && FUseChunkedTransfer);
        
        FHTTPSServer->SendResponseHeaders(FCurrentRequest, FCurrentResponse);
        
        if (FFile) {
            if (CompressionEnabled && (!FCurrentResponse->get_UseChunkedTransfer())) {
                // send manually compressed response
                FHTTPSServer->SendResponseData(MS);
                MS->Free();
            }
            else
                // send response as is; if both compression and chunked encoding are enabled
                // then server will compress data internally
                FHTTPSServer->SendResponseData(FFile);
            
            FFile->Free();
            FFile = NULL;
        }
        
        FHTTPSServer->ResponseComplete();
        
        if (!UseKeepAlive) {
            // close the connection if keep-alive was not requested
            FHTTPSServer->Close(false);
            FSocket->Close(false);
        }
    
    }
    catch (SBException E) {}
}

void THTTPSServerThread::Log(std::string Msg)
{
    if ((FOnLog) && (Msg != "")) FOnLog(Msg);
}

void THTTPSServerThread::Execute()
{
    FHTTPSServer->Open();
    
    while ((!sTerminated) && (FSocket->get_State() == issConnected))
    {
        try {
            if ((FSocket->CanReceive(500)) && (!sTerminated)) FHTTPSServer->DataAvailable();
            else std::this_thread::sleep_for(std::chrono::milliseconds(50));//Sleep(50);
        }
        catch (SBException E) {
            break;
        }
    }
    
    try {
        if ((new TElSSLClass(FHTTPSServer->GetControl(), false))->get_Active())
            FHTTPSServer->Close(FSocket->get_State() != issConnected);
        
        if (FSocket->get_State() == issConnected) FSocket->Close(false);
    }
    catch (SBException E) {}
    sFinished = true;
}

void SB_CALLBACK DoBeforeWebSocketServerUsed(void * _ObjectData, TObjectHandle Sender, TElWebSocketServerHandle WSServer)
{
    if (_ObjectData) {
        (new TElWebSocketServer(WSServer, false))->set_OnTextData(DoTextData, _ObjectData);
    }
}

void SB_CALLBACK DoWebSocketConnectionEstablished(void * _ObjectData, TObjectHandle Sender, TElWebSocketServerHandle WSServer)
{
    if (_ObjectData) {
        ((THTTPSServerThread*)_ObjectData)->FWSServer = new TElWebSocketServer(WSServer, false);
    }
}

void SB_CALLBACK DoTextData(void * _ObjectData, TObjectHandle Sender, const char * pcText, int32_t szText, int8_t Last)
{
    if (_ObjectData) {
        std::string Text(pcText, szText);
        ((THTTPSServerThread*)_ObjectData)->Log("WebSocket text received = " + Text);
        try {
            ((THTTPSServerThread*)_ObjectData)->FWSServer->SendText("Response: " + Text);
        }
        catch (SBException E) {}
    }
}

void SB_CALLBACK DoRequestHeadersReceived(void * _ObjectData, TObjectHandle Sender, TElHTTPServerRequestParamsHandle RequestInfo, int8_t * Accepted)
{
    if (_ObjectData) {
        try {
            ((THTTPSServerThread*)_ObjectData)->FCurrentRequest = new TElHTTPServerRequestParams(RequestInfo, false);
            ((THTTPSServerThread*)_ObjectData)->FCurrentResponse = new TElHTTPServerResponseParams(); // will be freed inside FHTTPSServer->ResponseComplete
            std::string reqURL;
            ((THTTPSServerThread*)_ObjectData)->FCurrentRequest->get_URL(reqURL);
            ((THTTPSServerThread*)_ObjectData)->FCurrentRequest->set_URL(URLExtractPath(reqURL));
            ((THTTPSServerThread*)_ObjectData)->FUpload = false;
            ((THTTPSServerThread*)_ObjectData)->FUploadStream->Clear();
            
            // try to load index file
            if (reqURL == "/") ((THTTPSServerThread*)_ObjectData)->FCurrentRequest->set_URL("/index.html");
            
            ((THTTPSServerThread*)_ObjectData)->FUpload = (reqURL == "/upload.html"); // upload URL is used
            
            if (((THTTPSServerThread*)_ObjectData)->FOnRequest)
                ((THTTPSServerThread*)_ObjectData)->FOnRequest(((THTTPSServerThread*)_ObjectData)->FCurrentRequest);
            
            if ((((THTTPSServerThread*)_ObjectData)->FCurrentRequest->get_ContentLength() <= 0) && (!((THTTPSServerThread*)_ObjectData)->FCurrentRequest->get_Chunked()) && (!((THTTPSServerThread*)_ObjectData)->FCurrentRequest->get_WebSocketRequest()))
                ((THTTPSServerThread*)_ObjectData)->ProcessRequest();
            
            *Accepted = true;
        }
        catch (SBException E) {
            *Accepted = false;
        }
    }
}

void SB_CALLBACK DoData(void * _ObjectData, TObjectHandle Sender, void * Buffer, int32_t Size)
{
    if (_ObjectData) {
        try {
            if (((THTTPSServerThread*)_ObjectData)->FUpload)
                ((THTTPSServerThread*)_ObjectData)->FUploadStream->Write(Buffer, Size);
        }
        catch (SBException E) {}
    }
}

void SB_CALLBACK DoRequestBodyReceived(void * _ObjectData, TObjectHandle Sender, TElHTTPServerRequestParamsHandle RequestInfo)
{
    if (_ObjectData) {
        ((THTTPSServerThread*)_ObjectData)->ProcessRequest();
    }
}

void SB_CALLBACK DoOpenConnection(void * _ObjectData, TObjectHandle Sender)
{
    if (_ObjectData) {
        ((THTTPSServerThread*)_ObjectData)->Log("Connection opened");
    }
}

void SB_CALLBACK DoError(void * _ObjectData, TObjectHandle Sender, int32_t ErrorCode, int8_t Fatal, int8_t Remote)
{
    if (_ObjectData) {
        ((THTTPSServerThread*)_ObjectData)->Log("SSL error: " + std::to_string(ErrorCode) + ", fatal: " + BoolToStr(Fatal) + ", remote: " + BoolToStr(Remote));
    }
}

void SB_CALLBACK DoSend(void * _ObjectData, TObjectHandle Sender, void * Buffer, int32_t Size)
{
    if (_ObjectData) {
        try {
            if (((THTTPSServerThread*)_ObjectData)->FSocket) {
                int Sent, Err;
                uint8_t *Buf = static_cast<uint8_t*>(Buffer);

                while ((Size > 0) && (((THTTPSServerThread*)_ObjectData)->FSocket->get_State() == issConnected))
                {
                    Err = ((THTTPSServerThread*)_ObjectData)->FSocket->Send(Buf, Size, Sent);

                    Size -= Sent;
					Buf += Sent;

                    if (Err == 0) {
                        //
                    }
                    else if (Err == 10035) {
                        std::this_thread::sleep_for(std::chrono::milliseconds(10));
                        continue;
                    }
                    else break;
                }
            }
        }
        catch (SBException E) {}
    }
}

void SB_CALLBACK DoReceive(void * _ObjectData, TObjectHandle Sender, void * Buffer, int32_t MaxSize, int32_t * Written)
{
    if (_ObjectData) {
        try {
            if ((!((THTTPSServerThread*)_ObjectData)->FSocket) || (((THTTPSServerThread*)_ObjectData)->FSocket->Receive(Buffer, MaxSize, *Written) != 0)) *Written = 0;
        }
        catch (SBException E) {
            *Written = 0;
        }
        
        try {
            if (*Written == 0) ((THTTPSServerThread*)_ObjectData)->FSocket->Close(false);
        }
        catch (SBException E) {}
    }
}

void SB_CALLBACK DoCloseConnection(void * _ObjectData, TObjectHandle Sender, TSBCloseReasonRaw CloseReason)
{
    if (_ObjectData) {
        try {
            if ((((THTTPSServerThread*)_ObjectData)->FSocket) && (((THTTPSServerThread*)_ObjectData)->FSocket->get_State() == issConnected)) ((THTTPSServerThread*)_ObjectData)->FSocket->Close(false);
            
            ((THTTPSServerThread*)_ObjectData)->Log("Connection closed: " + std::to_string(CloseReason));
        }
        catch (SBException E) {}
    }
}

