#include <cstring>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <algorithm>
#include <ctime>

#include "sbutils.h"
#include "sbcades.h"
#include "sbcms.h"
#include "sbcmsutils.h"
#include "sbx509.h"
#include "sbcrl.h"
#include "sbocspcommon.h"
#include "sbocspclient.h"
#include "sbtypes.h"
#include "sbconstants.h"
#include "sbcustomcertstorage.h"
#include "sbhttpsclient.h"
#include "sbcertvalidator.h"
#include "sbhttptspclient.h"
#include "sbmisc.h"

using namespace SecureBlackbox;

#define SB_SIG_LEVEL_BES	0
#define SB_SIG_LEVEL_T		1
#define SB_SIG_LEVEL_C		2
#define SB_SIG_LEVEL_X1		3
#define SB_SIG_LEVEL_X2		4
#define SB_SIG_LEVEL_XL1	5
#define SB_SIG_LEVEL_XL2	6
#define SB_SIG_LEVEL_A		7

TElX509CertificateValidator *CertificateValidator = NULL;

std::string UnQuote(std::string s)
{
	if (s.length() < 1)
		return s;

	s.erase(remove( s.begin(), s.end(), '\"' ), s.end());

	return s;
}

int SignatureLevel(std::string value)
{
	std::transform(value.begin(), value.end(), value.begin(), ::toupper);
	if (value == "CADES-BES")
		return SB_SIG_LEVEL_BES;
	else if (value == "CADES-T")
		return SB_SIG_LEVEL_T;
	else if (value == "CADES-C")
		return SB_SIG_LEVEL_C;
	else if (value == "CADES-X1")
		return SB_SIG_LEVEL_X1;
	else if (value == "CADES-X2")
		return SB_SIG_LEVEL_X2;
	else if (value == "CADES-XL1")
		return SB_SIG_LEVEL_XL1;
	else if (value == "CADES-XL2")
		return SB_SIG_LEVEL_XL2;
	else if (value == "CADES-A")
		return SB_SIG_LEVEL_A;
	else
		return -1;
}

std::string X509ValidityToString(TSBCertificateValidity Validity)
{
	std::string s;
	switch (Validity)
	{
	case cvOk:
		s = "OK";
		break;
	case cvSelfSigned:
		s = "Self-signed";
		break;
	case cvInvalid:
		s = "Invalid";
		break;
	case cvStorageError:
		s = "Storage error";
		break;
	case cvChainUnvalidated:
		s = "Chain unvalidated";
		break;
	default:
		s = "Unknown";
		break;
	}
	return s;
}

std::string	X509ReasonToString(TSBCertificateValidityReason Reason)
{
	std::string s;
	if ((f_vrBadData & Reason) > 0)
		s = "Bad data, ";
	if ((f_vrRevoked & Reason) > 0)
		s += "Revoked, ";
	if ((f_vrNotYetValid & Reason) > 0)
		s += "Not yet valid, ";
	if ((f_vrExpired & Reason) > 0)
		s += "Expired, ";
	if ((f_vrInvalidSignature & Reason) > 0)
		s += "Invalid signature, ";
	if ((f_vrUnknownCA & Reason) > 0)
		s += "Unknown CA, ";
	if ((f_vrCAUnauthorized & Reason) > 0)
		s += "CA Unauthorized, ";
	if ((f_vrCRLNotVerified & Reason) > 0)
		s += "CRL not verified, ";
	if ((f_vrOCSPNotVerified & Reason) > 0)
		s += "OCSP not verified, ";
	if ((f_vrIdentityMismatch & Reason) > 0)
		s += "Identity mismatch, ";
	if ((f_vrNoKeyUsage & Reason) > 0)
		s += "No key usage, ";
	if ((f_vrBlocked & Reason) > 0)
		s += "Blocked, ";

	if (s != "")
		s = s.substr(0, s.length() - 2);
	return s;
}

void SB_CALLBACK CertificateValidateEvent(void * _ObjectData, TObjectHandle Sender, TElX509CertificateHandle hX509Certificate, int8_t * Validate)
{
	TElX509Certificate Certificate (hX509Certificate, false);
	TSBCertificateValidity Validity = cvInvalid;
	TSBCertificateValidityReason Reason;
	if ((Certificate.get_Chain() == NULL) || (Certificate.get_Chain()->get_Certificates(0) == &Certificate))
	{
		// For proper CRL and OCSP validation please read instructions in 
		// description of ElX509CertificateValidator class in the help file
		std::string remoteHost, remoteIP;
		
		// Note that _ObjectData is set when the event handler is assigned
		TElHTTPSClient* HTTPSClient = (TElHTTPSClient *) _ObjectData;

		HTTPSClient->get_RemoteHost(remoteHost);
		HTTPSClient->get_RemoteIP(remoteIP);
		std::time_t t = std::time(0);
		CertificateValidator->ValidateForSSL(&Certificate, remoteHost, remoteIP, hrServer, NULL, false, t, Validity, Reason);
		*Validate = (Validity == cvOk) || (Validity == cvSelfSigned);
	}
	else		
		*Validate = true; 
}

void SB_CALLBACK HandleAfterCertificateValidation(void * /*_ObjectData*/, TObjectHandle /*hSender*/, TElX509CertificateHandle hCertificate, TElX509CertificateHandle hCACertificate, TSBCertificateValidityRaw *Validity, TSBCertificateValidityReasonRaw *Reason, int8_t *DoContinue)
{
	TElX509Certificate Certificate(hCertificate, false);
	TName name;
	Certificate.get_SubjectName(name);
	std::cout << "Validation done for " << (char *)name.CommonName <<
		", validity: " << X509ValidityToString((TSBCertificateValidity)*Validity) <<
		", reason: " << X509ReasonToString((TSBCertificateValidityReason)*Reason) << std::endl;
}

void SB_CALLBACK HandleBeforeCertificateValidation(void * /*_ObjectData*/, TObjectHandle /*hSender*/, TElX509CertificateHandle hCertificate)
{
	TElX509Certificate Certificate(hCertificate, false);
	TName name;
	Certificate.get_SubjectName(name);
	std::cout << "Validating certificate: " << (char *)name.CommonName << std::endl;
}

void SB_CALLBACK CertValidatorPreparedEvent(void * /*_ObjectData*/, TObjectHandle /*hSender*/, TElX509CertificateValidatorHandle * hCertValidator, TElX509CertificateHandle hCert)
{
	TElX509Certificate Cert(hCert, false);
	std::string s;
	Cert.get_SubjectRDN()->SaveToDNString(s);
	std::cout << "Certificate validator prepared for validation of " << s << std::endl;
	TElX509CertificateValidator certValidator (*hCertValidator, false);
	certValidator.set_OnAfterCertificateValidation(&HandleAfterCertificateValidation, NULL);
	certValidator.set_OnBeforeCertificateValidation(&HandleBeforeCertificateValidation, NULL);
	//certValidator.set_CheckCRL(false);
	certValidator.set_MandatoryCRLCheck(false);
	certValidator.set_MandatoryOCSPCheck(false);
	certValidator.set_MandatoryRevocationCheck(true);
}

void SB_CALLBACK CertValidatorFinishedEvent(void * /*_ObjectData*/, TObjectHandle /*Sender*/, TElX509CertificateValidatorHandle /*hCertValidator*/, TElX509CertificateHandle hCert, TSBCertificateValidityRaw Validity, TSBCertificateValidityReasonRaw Reason)
{
	TElX509Certificate Cert(hCert, false);
	std::string s;
	Cert.get_SubjectRDN()->SaveToDNString(s);
	std::cout << "Certificate validator finished validating " << s << 
		", validity: " << X509ValidityToString((TSBCertificateValidity)Validity) << 
		", reason: " << X509ReasonToString((TSBCertificateValidityReason)Reason) << std::endl;
}

int main(int argc, char **argv)
{
	std::string helpMessage = "consoleCAdES.exe [params]\n" \
		"[params]\n" \
		"-in <filename> : input file;\n" \
		"-out <filename> : output file;\n" \
		"-c <filename> <password> : certificate file and password if needed;\n" \
		"-trustedcerts <filename> : trusted certificate storage (*.pfx; *.p12);\n" \
		"-url <url> : timestamping server (TSA) server (default : empty);\n" \
		"-level <CADES-BES|CADES-T|CADES-C|CADES-X1|CADES-X2|CADES-XL1|CADES-XL2|CADES-A> : signature level (default : CADES-BES);\n" \
		"-h : help;\n";
	std::string inputFilename, outputFilename, certFilename, certPassword, trustedCertsFilename;
	std::string level = "CAdES-BES";
	std::string url;

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << E.what() << std::endl;
	}

	if (argc < 2)
	{
		std::cout << helpMessage << std::endl;
		return 0;
	}

	try
	{
		int i = 1;
		while (i < argc)
		{
			std::string param;
			param.assign(argv[i]);

			if (param == "-in")
			{
				if (++i < argc)
					inputFilename = UnQuote(argv[i]);
			}
			else if (param == "-out")
			{
				if (++i < argc)
					outputFilename = UnQuote(argv[i]);
			}
			else if (param == "-url")
			{
				if (++i < argc)
					url = UnQuote(argv[i]);
			}
			else if (param == "-c")
			{
				if (++i < argc)
					certFilename = UnQuote(argv[i]);
				if (++i < argc)
				{
					if (argv[i][0] != '-')
						certPassword = UnQuote(argv[i]);
					else
						i--;
				}
			}
			else if (param == "-trustedcerts")
			{
				if (++i < argc)
					trustedCertsFilename = UnQuote(argv[i]);
			}
			else if (param == "-level")
			{
				if (++i < argc)
					level = UnQuote(argv[i]);
			}
			else if (param == "-h")
			{
				std::cout << helpMessage << std::endl;
			}

			i++;
		}
	}
	catch (...)
	{
		std::cout << "Failed to parse parameters" << std::endl;
		std::cout << helpMessage << std::endl;
		return 0;
	}

	try
	{
		TElSignedCMSMessage cms(NULL);
		TElCAdESSignatureProcessor processor;
		TElMemoryCertStorage trustedCerts (NULL);
		TElX509Certificate cert (NULL);
		TElHTTPSClient httpClient(NULL);
		TElHTTPTSPClient tspClient(NULL);
		CertificateValidator = new TElX509CertificateValidator(NULL);

		processor.set_OnCertValidatorPrepared(&CertValidatorPreparedEvent, NULL);
		processor.set_OnCertValidatorFinished(&CertValidatorFinishedEvent, NULL);

		httpClient.set_OnCertificateValidate(&CertificateValidateEvent, &httpClient);

		int res = cert.LoadFromFileAuto(certFilename, certPassword);
		if (res != 0)
		{
			std::cout << std::hex << "Failed to load certificate, error: 0x" << res << std::endl;
			return 0;
		}

		if (trustedCertsFilename != "")
		{
			TFileStream fs(trustedCertsFilename, fmOpenRead);
			res = trustedCerts.LoadFromStreamPFX(fs, "", 0);
			if (res != 0)
			{
				std::cout << std::hex << "Failed to load certificate storage: 0x" << res << std::endl;
				return 0;
			}

			std::cout << "Trusted certificates loaded: " << trustedCerts.get_Count() << std::endl;
		}

		processor.set_TrustedCertificates(trustedCerts);

		TFileStream inputStream (inputFilename, filemodeOpenRead);

		cms.CreateNew(inputStream, 0, inputStream.get_Size());

		TElCMSSignature* sig = cms.get_Signatures(cms.AddSignature());

		processor.set_Signature(sig);
		processor.set_ForceSigningCertificateV2(true);
		processor.set_IgnoreChainValidationErrors(true);
		processor.set_OfflineMode(false);

		int sigLevel = SignatureLevel(level);

		if (sigLevel >= SB_SIG_LEVEL_T)
		{
			if (url == "")
			{
				std::cout << "Timestamp URL is not set" << std::endl;
				return 0;
			}

			tspClient.set_HTTPClient(httpClient);
			tspClient.set_URL(url);
		}

		switch (sigLevel)
		{
			case SB_SIG_LEVEL_BES : 
				processor.CreateBES(cert);
				break;
			case SB_SIG_LEVEL_T : 
				processor.CreateT(cert, tspClient);
				break;
			case SB_SIG_LEVEL_C :
				processor.CreateC(cert, tspClient);
				break;
			case SB_SIG_LEVEL_X1 : 
				processor.CreateXType1(cert, tspClient, tspClient);
				break;
			case SB_SIG_LEVEL_X2 : 
				processor.CreateXType2(cert, tspClient, tspClient);
				break;
			case SB_SIG_LEVEL_XL1 : 
				processor.CreateXLType1(cert, tspClient, tspClient);
				break;
			case SB_SIG_LEVEL_XL2 : 
				processor.CreateXLType2(cert, tspClient, tspClient);
				break;
			case SB_SIG_LEVEL_A :
				{
					std::cout << "Creation of A signatures is not supported. Please create a T/C/XL signature and then upgrade it to A." << std::endl;
					return 0;
				}
			default:
				{
					std::cout << "Unsupported signature level (" << sigLevel << ")" << std::endl;
					return 0;
				}
		}

		std::cout << "Signing succeeded" << std::endl;

		try
		{
			TFileStream outputStream(outputFilename, filemodeCreate);
			cms.Save(outputStream);
		}
		catch (SBException E)
		{
			std::cout << "Failed to save CMS: " << E.what() << std::endl;
			return 0;
		}

		delete(CertificateValidator);
	}
	catch (SBException E)
	{
		std::cout << "Error:" << E.what() << std::endl;
		return 0;
	}

	return 0;
}