#include <cstring>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>

#include "sbsshcommon.h"
#include "sbsshkeystorage.h"
#include "sbsshforwarding.h"
#include "sbsocket.h"
#include "sbutils.h"

using namespace SecureBlackbox;

void SB_CALLBACK Forwarding_OnKeyValidate(void * /* objData */, TObjectHandle /* hSender */, TElSSHKeyHandle hServerKey, int8_t * Validate)
{
	try
	{
		TElSSHKey ServerKey(hServerKey, false);
		TMessageDigest128 M128;
		ServerKey.get_FingerprintMD5(M128);
		std::string dgst;
		DigestToStr(M128, true, dgst);
		std::cout << "Server key received, fingerprint " << dgst << std::endl;
		*Validate = true; // NEVER do this. You MUST check the key validity somehow
	}
	catch (SBException E)
	{
		std::cout << "Unexpected error in OnKeyValidate event handler" << std::endl;
		std::cout << E.what() << std::endl;
		std::cout << "Stack trace: " << E.getErrorStackTrace().c_str() << std::endl;
		*Validate = false;
	}
}

void SB_CALLBACK Forwarding_AuthenticationSuccess(void * /* objData */, TObjectHandle /* hSender */)
{
	std::cout << "Authentication succeeded" << std::endl;
}

void SB_CALLBACK Forwarding_AuthenticationFailed(void * /* objData */, TObjectHandle /* hSender */, int32_t AuthenticationType)
{
	std::cout << "Authentication type " << AuthenticationType << " failed" << std::endl;
}

void SB_CALLBACK Forwarding_Error(void * /* objData */, TObjectHandle hSender, int32_t ErrorCode)
{
	std::cout << "SSH protocol error " << ErrorCode << std::endl;

	std::cout << "If you have ensured that all connection parameters are correct and you still can't connect," << std::endl;
	std::cout << "please contact EldoS support as described on http://www.eldos.com/sbb/support-tech.php" << std::endl;
	std::cout << "Remember to provide details about the error that happened." << std::endl;

	try
	{
		std::string sname = "";
		TElSSHLocalPortForwarding Forwarding(hSender, ohFalse);
		Forwarding.get_ServerSoftwareName(sname);
		if (sname.length() > 0)
			std::cout << "Server software identified itself as: " << sname.c_str() << std::endl;
	}
	catch (SBException E)
	{
		std::cout << "Unexpected error in OnError event handler" << std::endl;
		std::cout << E.what() << std::endl;
		std::cout << "Stack trace: " << E.getErrorStackTrace().c_str() << std::endl;
	}
}

void LogConnection(TElSSHForwardedConnectionHandle hConn)
{
	// create proxy class from handle
	TElSSHForwardedConnection Conn(hConn, ohFalse);
	std::cout << "  Host: ";
	if (Conn.get_Socket() != NULL)
	{
		std::string Addr;
		Conn.get_Socket()->get_Address(Addr);
		std::cout << Addr;
	}
	else
		std::cout << "N/A";

	std::cout << ", Socket: ";
	switch (Conn.get_SocketState())
	{
	case fssEstablishing:
		std::cout << "Establishing";
		break;
	case fssActive:
		std::cout << "Active";
		break;
	case fssClosing:
		std::cout << "Closing";
		break;
	case fssClosed:
		std::cout << "Closed";
		break;
	}

	std::cout << " (R: " << Conn.get_ReceivedFromSocket() << ", S: " << Conn.get_SentToSocket() << ")";
	std::cout << ", Channel: ";
	switch (Conn.get_ChannelState())
	{
	case fcsEstablishing:
		std::cout << "Establishing";
		break;
	case fcsActive:
		std::cout << "Active";
		break;
	case fcsClosing:
		std::cout << "Closing";
		break;
	case fcsClosed:
		std::cout << "Closed";
		break;
	}

	std::cout << " (R: " << Conn.get_ReceivedFromChannel() << ", S: " << Conn.get_SentToChannel() << ")";
	std::cout << std::endl;
}

void SB_CALLBACK Forwarding_ConnectionOpen(void * /* objData */, TObjectHandle /* hSender */, TElSSHForwardedConnectionHandle hConn)
{
	std::cout << "New secure channel opened" << std::endl;
	LogConnection(hConn);
}

void SB_CALLBACK Forwarding_ConnectionChange(void * /* objData */, TObjectHandle /* hSender */, TElSSHForwardedConnectionHandle hConn)
{
	std::cout << "Secure channel changed" << std::endl;
	LogConnection(hConn);
}

void SB_CALLBACK Forwarding_ConnectionClose(void * /* objData */, TObjectHandle /* hSender */, TElSSHForwardedConnectionHandle hConn)
{
	std::cout << "Secure channel closed" << std::endl;
	LogConnection(hConn);
}

void SB_CALLBACK Forwarding_ConnectionError(void * /* objData */, TObjectHandle /* hSender */, TElSSHForwardedConnectionHandle /* hConn */, int32_t ErrorCode)
{
	std::cout << "SSH connection error: " << ErrorCode << std::endl;
}

void SB_CALLBACK Forwarding_Open(void * /* objData */, TObjectHandle /* hSender */)
{
	std::cout << "SSH connection established" << std::endl;
}

void SB_CALLBACK Forwarding_Close(void * /* objData */, TObjectHandle /* hSender */)
{
	std::cout << "SSH connection closed" << std::endl;
}

int main(int argc, char **argv) {

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << std::endl;
		std::cout << E.what() << std::endl;
		return 0;
	}

	if (argc < 3)
	{
		std::cout << "Usage:" << std::endl;
		std::cout << "ConsoleLocalPortForward -addr[ess] <address> [-port <port>] -user[name] <user> -pass[word] <password> [-fwd-host <forwarded host>] -fwd-port <forwarded port> -dest-host <destination host> -dest-port <destination port>" << std::endl;
		return 0;
	}

	int paramPort = 22;
	int paramDestPort = 80;
	int paramForwardedPort = 5001;
	std::string paramAddress;
	std::string paramUsername;
	std::string paramPassword;
	std::string paramDestHost("127.0.0.1");
	std::string paramForwardedHost;

	int i = 1;
	while (i < argc - 1)
	{
		char *arg = argv[i];
		if (!strcmp(arg, "-addr") ||
			!strcmp(arg, "-address"))
			paramAddress.assign(argv[++i]);
		else if (!strcmp(arg, "-port"))
		{
			paramPort = atoi(argv[++i]);
			if (0 == paramPort)
				paramPort = 22;
		}
		else if (!strcmp(arg, "-user") ||
			!strcmp(arg, "-username"))
			paramUsername.assign(argv[++i]);
		else if (!strcmp(arg, "-pass") ||
			!strcmp(arg, "-password"))
			paramPassword.assign(argv[++i]);
		else if (!strcmp(arg, "-dest-host"))
			paramDestHost.assign(argv[++i]);
		else if (!strcmp(arg, "-dest-port"))
		{
			paramDestPort = atoi(argv[++i]);
			if (0 == paramDestPort)
				paramDestPort = 22;
		}
		else if (!strcmp(arg, "-fwd-host"))
			paramDestHost.assign(argv[++i]);
		else if (!strcmp(arg, "-fwd-port"))
		{
			paramForwardedPort = atoi(argv[++i]);
			if (0 == paramForwardedPort)
				paramForwardedPort = 22;
		}
		else
			std::cout << "Unknown parameter: " << arg << std::endl;

		i++;
	}

	TElSSHLocalPortForwarding Forwarding(NULL);
	try
	{
		Forwarding.set_SynchronizeGUI(false);
		Forwarding.set_Address(paramAddress);
		Forwarding.set_Port(paramPort);

		Forwarding.set_Username(paramUsername);
		Forwarding.set_Password(paramPassword);

		Forwarding.set_ForwardedPort(paramForwardedPort);
		Forwarding.set_ForwardedHost(paramForwardedHost);
		Forwarding.set_DestPort(paramDestPort);
		Forwarding.set_DestHost(paramDestHost);

		Forwarding.set_OnKeyValidate(&Forwarding_OnKeyValidate, NULL);
		Forwarding.set_OnAuthenticationSuccess(&Forwarding_AuthenticationSuccess, NULL);
		Forwarding.set_OnAuthenticationFailed(&Forwarding_AuthenticationFailed, NULL);
		Forwarding.set_OnError(&Forwarding_Error, NULL);
		Forwarding.set_OnOpen(&Forwarding_Open, NULL);
		Forwarding.set_OnClose(&Forwarding_Close, NULL);

		Forwarding.set_OnConnectionOpen(&Forwarding_ConnectionOpen, NULL);
		Forwarding.set_OnConnectionChange(&Forwarding_ConnectionChange, NULL);
		Forwarding.set_OnConnectionClose(&Forwarding_ConnectionClose, NULL);
		Forwarding.set_OnConnectionError(&Forwarding_ConnectionError, NULL);

		std::cout << "Press enter key to close connection and exit" << std::endl;
		std::cout << "Forwarding " << paramForwardedHost << ":" << paramForwardedPort << " to " << paramDestHost << ":" << paramDestPort << std::endl;
		std::cout << "Establishing connection on " << paramAddress << ":" << paramPort << std::endl;

		/*
		// Sample code to create additional tunnel
		int32_t tunnelIdx = Forwarding.AddTunnel();
		TElSSHForwardingTunnel* Tunnel2 = Forwarding.get_Tunnels(tunnelIdx);
		Tunnel2->set_ForwardedPort(paramForwardedPort + 1);
		Tunnel2->set_ForwardedHost(paramForwardedHost);
		Tunnel2->set_DestPort(paramDestPort);
		Tunnel2->set_DestHost(paramDestHost);
		*/

		Forwarding.Open();

		// forwarding until enter key is pressed
		fgetc(stdin);

		std::cout << "Closing connection" << std::endl;
		Forwarding.Close();
	}
	catch (SBException E)
	{
		std::cout << "Connection failed due to exception:" << std::endl;
		std::cout << E.what() << std::endl;
		std::cout << "Stack trace: " << E.getErrorStackTrace().c_str() << std::endl;

		std::cout << "If you have ensured that all connection parameters are correct and you still can't connect," << std::endl;
		std::cout << "please contact EldoS support as described on http://www.eldos.com/sbb/support-tech.php" << std::endl;
		std::cout << "Remember to provide details about the error that happened." << std::endl;

		std::string sname = "";
		Forwarding.get_ServerSoftwareName(sname);
		if (sname.length() > 0)
			std::cout << "Server software identified itself as: " << sname.c_str() << std::endl;
	}

	fflush(stdout);
	return 0;
}
