@echo off

if exist C:\cygwin\bin\*.* set CYGWIN_PATH=C:\cygwin\bin
if exist D:\cygwin\bin\*.* set CYGWIN_PATH=D:\cygwin\bin

if exist C:\MinGW\bin\*.* set MINGW_PATH=C:\MinGW\bin
if exist D:\MinGW\bin\*.* set MINGW_PATH=D:\MinGW\bin

rem Test if Visual Studio 2005 is installed
if exist "%ProgramFiles%\Microsoft Visual Studio 8\VC\*.*" set VC_PATH=%ProgramFiles%\Microsoft Visual Studio 8\VC\
if exist "%ProgramFiles(x86)%\Microsoft Visual Studio 8\VC\*.*" set VC_PATH=%ProgramFiles(x86)%\Microsoft Visual Studio 8\VC\

rem Test if Visual Studio 2008 is installed
if exist "%ProgramFiles%\Microsoft Visual Studio 9.0\VC\*.*" set VC_PATH=%ProgramFiles%\Microsoft Visual Studio 9.0\VC\
if exist "%ProgramFiles(x86)%\Microsoft Visual Studio 9.0\VC\*.*" set VC_PATH=%ProgramFiles(x86)%\Microsoft Visual Studio 9.0\VC\

rem Test if Visual Studio 2010 is installed
if exist "%ProgramFiles%\Microsoft Visual Studio 10.0\VC\*.*" set VC_PATH=%ProgramFiles%\Microsoft Visual Studio 10.0\VC\
if exist "%ProgramFiles(x86)%\Microsoft Visual Studio 10.0\VC\*.*" set VC_PATH=%ProgramFiles(x86)%\Microsoft Visual Studio 10.0\VC\

rem Test if Visual Studio 2012 is installed
if exist "%ProgramFiles%\Microsoft Visual Studio 11.0\VC\*.*" set VC_PATH=%ProgramFiles%\Microsoft Visual Studio 11.0\VC\
if exist "%ProgramFiles(x86)%\Microsoft Visual Studio 11.0\VC\*.*" set VC_PATH=%ProgramFiles(x86)%\Microsoft Visual Studio 11.0\VC\

rem Test if Visual Studio 2013 is installed
if exist "%ProgramFiles%\Microsoft Visual Studio 12.0\VC\*.*" set VC_PATH=%ProgramFiles%\Microsoft Visual Studio 12.0\VC\
if exist "%ProgramFiles(x86)%\Microsoft Visual Studio 12.0\VC\*.*" set VC_PATH=%ProgramFiles(x86)%\Microsoft Visual Studio 12.0\VC\

rem Test if Visual Studio 2015 is installed
if exist "%ProgramFiles%\Microsoft Visual Studio 14.0\VC\*.*" set VC_PATH=%ProgramFiles%\Microsoft Visual Studio 14.0\VC\
if exist "%ProgramFiles(x86)%\Microsoft Visual Studio 14.0\VC\*.*" set VC_PATH=%ProgramFiles(x86)%\Microsoft Visual Studio 14.0\VC\

@REM -----------------------------------------------------------------------

if "%1"=="g++-3" goto gpp3
if "%1"=="g3" goto gpp3
if "%1"=="g++-4" goto gpp4
if "%1"=="g4" goto gpp4
if "%1"=="msvc" goto msvc
if "%1"=="vc" goto msvc
if "%1"=="mgw" goto mgw

echo Build static libraries
echo Usage: Makefile.cmd compiler [target] [config]
echo  g3, g++-3 - GNU 3 C++ compiler
echo  g4, g++-4 - GNU 4 C++ compiler (targets: x86, x64)
echo  mgw - MinGW C++ compiler
echo  vc, msvc - Microsoft Visual C++ compiler (targets: x86, amd64, x86_amd64; config: release, debug)

pause

exit /B

@REM -----------------------------------------------------------------------

:gpp3

echo Compile static library using g++-3 compiler

set PATH=%CYGWIN_PATH%
set CYGWIN=nodosfilewarning 

set CPP=g++-3
set STATIC_LIB=sbbex3
set CUSTOM_FLAGS=
set CUSTOM_DEFINES=-DSB_DEFINE_WSTRING 

make -f Makefile.gpp

exit /B

:gpp4

echo Compile static library using g++-4 compiler

set PATH=%CYGWIN_PATH%
set CYGWIN=nodosfilewarning 

set CPP=g++-4
set STATIC_LIB=sbbex4
set CUSTOM_FLAGS=-m32
set CUSTOM_DEFINES=
if "%2"=="x64" (
  set CUSTOM_FLAGS=-m64
  set CUSTOM_DEFINES=-DCPU64
)

make -f Makefile.gpp

exit /B

@REM -----------------------------------------------------------------------

:mgw

echo Compile static library using MinGW g++ compiler

set PATH=%MINGW_PATH%

set CPP=g++
set STATIC_LIB=sbbexmgw
set CUSTOM_FLAGS=-m32
set CUSTOM_DEFINES=
if "%2"=="x64" (
  set CUSTOM_FLAGS=-m64
  set CUSTOM_DEFINES=-DCPU64
)

mingw32-make -f Makefile.mgw

exit /B

@REM -----------------------------------------------------------------------

:msvc

if "%VC_PATH%"=="" (
	echo Visual Studio has not been found
	echo please edit this batch file to explicitly specify VC_PATH variable value
	pause
	exit /b
)

echo Compile static library using VC compiler

call "%VC_PATH%\vcvarsall.bat" %2

set CUSTOM_DEFINES=
if "%2"=="amd64" set CUSTOM_DEFINES=/D CPU64
if "%2"=="x86_amd64" ( 
  set CUSTOM_DEFINES=/D CPU64
  set CUSTOM_LDFLAGS=/machine:x64
)

set CONFIG=
if "%3"=="debug" set CONFIG=DEBUG

nmake -f Makefile.vc

exit /B
