#!/bin/bash

export STATIC_LIB=sbbex

export CUSTOM_FLAGS=
export CUSTOM_DEFINES=

lowercase() {
  echo $1 | tr "[:upper:]" "[:lower:]" 
}

vercomp () {
    if [[ $1 == $2 ]]; then
        return 1
    fi
    local IFS=.
    local i ver1=($1) ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++)); do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++)); do
        if [[ -z ${ver2[i]} ]]; then
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]})); then
            return 2
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]})); then
            return 0
        fi
    done
    return 1
}

if [ "$1" == "g" -o "$1" == "g++" ]; then

  export CPP=g++

  # check if C++0x/C++11 features are supported
  export VER=`g++ -dumpversion`
  vercomp $VER "4.7"
  if [ $? -ge 1 ]; then
    export CUSTOM_FLAGS+=" -std=c++11"
  else
    vercomp $VER "4.4"
    if [ $? -ge 1 ]; then
      export CUSTOM_FLAGS+=" -std=c++0x"
    else
      export CUSTOM_DEFINES+=" -DSB_DEFINE_CHAR16"
    fi
  fi

  OS=`lowercase \`uname\``
  if [ "$OS" == "darwin" ]; then
    export CUSTOM_DEFINES+=" -DMACOS"
  else
    export CUSTOM_DEFINES+=" -DLINUX"
  fi

  if [ `uname -m` == 'x86_64' ]; then
    export CUSTOM_FLAGS+=" -fPIC"
    export CUSTOM_DEFINES+=" -DCPU64"
  fi

  make -f Makefile.gpp	

  exit

fi

if [ "$1" == "clang" -o "$1" == "clang++" ]; then

  export CPP=clang++

  # check if C++0x/C++11 features are supported
  export VER=`clang++ --version 2>&1 | grep version | awk '{print $4}'`
  vercomp $VER "3.1"
  if [ $? -ge 1 ]; then
    export CUSTOM_FLAGS+=" -std=c++11 -stdlib=libc++"
  else
    vercomp $VER "3.0"
    if [ $? -ge 1 ]; then
      export CUSTOM_FLAGS+=" -std=c++0x -stdlib=libc++"
    else
      export CUSTOM_DEFINES+=" -DSB_DEFINE_CHAR16"
    fi
  fi

  OS=`lowercase \`uname\``
  if [ "$OS" == "darwin" ]; then
    export CUSTOM_DEFINES+=" -DMACOS"
  else
    export CUSTOM_DEFINES+=" -DLINUX"
  fi

  if [ `uname -m` == 'x86_64' ]; then
    export CUSTOM_FLAGS+=" -fPIC"
    export CUSTOM_DEFINES+=" -DCPU64"
  fi

  make -f Makefile.gpp	

  exit

fi

echo Build static libraries
echo Usage: makefile.sh compiler 
echo  g, g++ - GNU C++ compiler
echo  clang, clang++ - LLVM compiler 

read input

if [[ $input != "" ]]; then
  echo Running: ./makefile.sh $input
  ./makefile.sh $input
fi

exit
